@@ -5,7 +5,7 @@ package Test::Mojibake;
 use strict;
 use warnings qw(all);
 
-our $VERSION = '1.1'; # VERSION
+our $VERSION = '1.2'; # VERSION
 
 use File::Spec::Functions;
 use Test::Builder;
@@ -27,11 +27,14 @@ my $Test = Test::Builder->new;
 
 # Use a faster/safer XS alternative, if present
 
-## no critic (ProhibitStringyEval, RequireCheckingReturnValueOfEval)
-eval 'require Unicode::CheckUTF8';
-
-## no critic (ProhibitPackageVars)
-our $use_xs = $@ ? 0 : 1;
+## no critic (ProhibitStringyEval, RequireCheckingReturnValueOfEval, ProhibitPackageVars)
+our ($use_xs, $use_pp) = (0, 0);
+if ( eval 'require Unicode::CheckUTF8' ) {
+    $use_xs = 1;
+}
+elsif ( eval 'require Unicode::CheckUTF8::PP' ) {
+    $use_pp = 1;
+}
 
 sub import {
     my ($self, @args) = @_;
@@ -244,6 +247,12 @@ sub _detect_utf8 {
         } else {
             return 0;
         }
+    } elsif ($use_pp) {
+        if (Unicode::CheckUTF8::PP::is_utf8(${$str})) {
+            return (${$str} =~ m{[\x{80}-\x{ff}]}x) ? 2 : 1
+        } else {
+            return 0;
+        }
     }
 
     my $d       = 0;
@@ -277,7 +286,7 @@ sub _detect_utf8 {
                 return 0;
             }
 
-            my @buf = ((0) x 4, $c & ((1 << (6 - $bits)) - 1));
+            my @buf = ((0) x 4, $c & ((1 << (7 - $bits)) - 1));
             while ($bits > 1) {
                 $i++;
                 $bv = ord(substr(${$str}, $i, 1));
@@ -311,16 +320,10 @@ Test::Mojibake - check your source for encoding misbehavior.
 
 =head1 VERSION
 
-version 1.1
+version 1.2
 
 =head1 SYNOPSIS
 
-    # Test::Mojibake lets you check for inconsistencies in source/documentation encoding, and report its results in standard Test::Simple fashion.
-
-    use Test::Mojibake;
-    file_encoding_ok($file, 'Valid encoding');
-    done_testing($num_tests);
-
 =head1 DESCRIPTION
 
 Many modern text editors automatically save files using UTF-8 codification, however, L<perl> interpreter does not expects it I<by default>. Whereas this does not represent a big deal on (most) backend-oriented programs, Web framework (L<Catalyst|http://www.catalystframework.org/>, L<Mojolicious|http://mojolicio.us/>) based applications will suffer of so-called L<Mojibake|http://en.wikipedia.org/wiki/Mojibake> (lit. "unintelligible sequence of characters").
@@ -419,6 +422,12 @@ L<Unicode::CheckUTF8> is highly recommended, however, it is optional and this fu
 
 =for test_synopsis my ($file, $num_tests);
 
+    # Test::Mojibake lets you check for inconsistencies in source/documentation encoding, and report its results in standard Test::Simple fashion.
+
+    use Test::Mojibake;
+    file_encoding_ok($file, 'Valid encoding');
+    done_testing($num_tests);
+
 =head1 SAMPLE TEST SCRIPT
 
 Module authors can include the following in a F<t/mojibake.t> file and have L<Test::Mojibake> automatically find and check all source files in a module distribution:
@@ -535,14 +544,14 @@ Stanislaw Pusep <stas@sysd.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Stanislaw Pusep.
+This software is copyright (c) 2017 by Stanislaw Pusep.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =head1 CONTRIBUTORS
 
-=for stopwords Dave Rolsky Karen Etheridge
+=for stopwords Dave Rolsky Hunter McMillen John SJ Anderson Karen Etheridge
 
 =over 4
 
@@ -552,6 +561,14 @@ Dave Rolsky <autarch@urth.org>
 
 =item *
 
+Hunter McMillen <mcmillhj@gmail.com>
+
+=item *
+
+John SJ Anderson <john@genehack.org>
+
+=item *
+
 Karen Etheridge <ether@cpan.org>
 
 =back
